/******************************************************************
*        PoLAR (Portable Library for Augmented Reality)           *
*     Copyright (C) 2015 Inria, Université de Lorraine, CNRS      *
*                                                                 *
* This library is free software; you can redistribute it and/or   *
* modify it under the terms of the GNU General Public License     *
* as published by the Free Software Foundation; either version 3  *
* of the License, or (at your option) any later version.          *
*                                                                 *
* This library is distributed in the hope that it will be useful, *
* but WITHOUT ANY WARRANTY; without even the implied warranty of  *
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the    *
* GNU General Public License for more details.                    *
*                                                                 *
*                                                                 *
* You should have received a copy of the GNU General Public       *
* License along with this library.                                *
* If not, see <http://www.gnu.org/licenses/>.                     *
******************************************************************/

#include <QApplication>

#include "basicInterface3Dbis.h"
#include "props.h"
#include <PoLAR/Image.h>
#include <PoLAR/Util.h>

#include <osg/Geode>
#include <osg/MatrixTransform>
#include <osg/Geometry>
#include <osg/ShapeDrawable>
#include <osg/StateSet>
#include <osg/ColorMask>
#include <osg/PolygonMode>
#include <osgDB/ReadFile>
#include <osgDB/WriteFile>
#include <osgUtil/SmoothingVisitor>
#include <osgUtil/Optimizer>
#include <osg/Texture2D>
#include <iostream>


int main(int argc,char ** argv)
{
    QApplication app(argc,argv);
    Interface inter;
    osg::ref_ptr<PoLAR::Image_uc> myImage;

    //osg::setNotifyLevel(osg::DEBUG_FP);


    
         myImage = new PoLAR::Image_uc("../photo.jpg",true);
        

        //Ajoute l'image en fond
        inter.addImage(myImage, 1);
        //Ma matrice de projection 
	osg::Matrixd P = PoLAR::Util::readProjectionMatrix("../projection.proj");
        //Le type de projection
        PoLAR::Viewer2D3D::ProjectionType pt = PoLAR::Viewer2D3D::VISION;
	inter.setProjection(P, pt);
        

  
                 //Declaration et construction de mon Sol à l'aide de Geode et de ShapeDrawable
		 osg::ref_ptr<osg::Geode> solforme = new osg::Geode;
           	 osg::ref_ptr<osg::ShapeDrawable> forme;
           	 forme = new osg::ShapeDrawable(new osg::Box(osg::Vec3(0.5f, 0.5f, 0.0f), 8.0f, 10.0f, 0.001f));
           	 solforme->addDrawable(forme.get());
                 //Creation a proprement parler de l'objet sol
           	 osg::ref_ptr<PoLAR::Object3D> sol = new PoLAR::Object3D(solforme.get());
                 //Associe un nom a l'objet 
                 sol->setName("Sol");
                 //Declare que l'objet sol est editable
           	 sol->setEditOn();
                 //Construction de ma matrices de transformation ici avec juste une translation selon X
		 osg::Matrixd matrice1 = osg::Matrixd::translate(3.5,0,0); 
           	 sol->setPhantomOn(); //Rend le sol invisible
                 //Applique les transformations a l'objet avec la matrice
 		 sol->setTransformationMatrix(matrice1);
                 //Ajout de l'objet a la fenêtre avec la fonction addNode qui permet le picking
           	 inter.addNode(sol);


                 //Déclaration et construction de mon mur de gauche
                 osg::ref_ptr<osg::Geode> formemur = new osg::Geode;
           	 osg::ref_ptr<osg::ShapeDrawable> forme2;
           	 forme2 = new osg::ShapeDrawable(new osg::Box(osg::Vec3(0.5f, 0.5f, 0.0f), 5.0f, 10.0f, 0.0f));
           	 formemur->addDrawable(forme2.get());
           	 osg::ref_ptr<PoLAR::Object3D> mur = new PoLAR::Object3D(formemur.get());
           	 mur->setName("Mur de gauche");
           	 mur->setEditOn();
                 //Cette fois-ci il y a egalement une rotation, on multiplie les deux matrices de transformations entre elles
		 osg::Matrixd matrice2 = osg::Matrixd::rotate(M_PI/2,0.0,1.0,0.0)* osg::Matrixd::translate(0,7,0); 
                 mur->setTransformationMatrix(matrice2);
                 mur->setPhantomOn(); //Rend le mur invisible
           	 inter.addNode(mur);


                //Ajout du chapeau de notre lapin
                osg::ref_ptr<PoLAR::Object3D> chapo = new PoLAR::Object3D("../chapo.obj", true, true, true);
                osg::Matrixd matrice3 = osg::Matrixd::rotate(M_PI/2,0.0,0.0,1.0) * osg::Matrixd::translate(53,95,13)* osg::Matrixd::scale(0.03,0.03,0.03); // initialisation of matrix m with a custom tranformation 50 130 0
                chapo->setName("Chapeau");
                chapo->setTransformationMatrix(matrice3);
                inter.addNode(chapo);
                //Permet le picking de notre objet
                inter.getViewer()->setTrackNode(chapo.get()); 
            
                //Ajout de la pipe
		osg::ref_ptr<PoLAR::Object3D> pipe = new PoLAR::Object3D("../pipe.obj",true,true,true);
		osg::Matrixd matrice4 = osg::Matrixd::rotate(M_PI/2,0.0,0.0,1.0) * osg::Matrixd::translate(15.1,26,1.7)* osg::Matrixd::scale(0.1,0.1,0.1);
            	pipe->setName("Pipe");
                pipe->setTransformationMatrix(matrice4);
                inter.addNode(pipe);
                inter.getViewer()->setTrackNode(pipe.get());

                //Ajout de notre lapin
        	osg::ref_ptr<PoLAR::Object3D> lapin = new PoLAR::Object3D("../bunny.obj",true,true,true);
		osg::Matrixd matrice5 = osg::Matrixd::rotate(2*M_PI/3,0.0,0.0,1.0) * osg::Matrixd::translate(12,21,-0.1)* osg::Matrixd::scale(0.15,0.15,0.15);
            	lapin->setName("Lapin");
                lapin->setTransformationMatrix(matrice5);
                inter.addNode(lapin);
                inter.getViewer()->setTrackNode(lapin.get());

        //Affiche la fenêtre
        inter.show();
    
    

    return app.exec();
}
